@@ -8,6 +8,13 @@
 
 NTL_START_IMPL
 
+static inline
+void CheckFinite(double *p)
+{
+   if (!IsFinite(p)) Error("G_LLL_FP: numbers too big...use G_LLL_XD");
+}
+
+
 
 static void RowTransform(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1)
 // x = x - y*MU
@@ -106,27 +113,37 @@ static void RowTransformFinish(vec_ZZ& A, double *a, long *in_a)
       }
       else {
          conv(a[i], A(i));
+         CheckFinite(&a[i]);
       }
    }
 }
 
 
 static void RowTransform(vec_ZZ& A, vec_ZZ& B, const ZZ& MU1, 
-                         double mu, double *a, double *b, long *in_a,
+                         double *a, double *b, long *in_a,
                          double& max_a, double max_b, long& in_float)
 // x = x - y*MU
 {
    static ZZ T, MU;
    long k;
+   double mu;
+
+   conv(mu, MU1);
+   CheckFinite(&mu);
 
    long n = A.length();
    long i;
 
    if (in_float) {
-      max_a += fabs(mu)*max_b;
-      if (max_a >= TR_BND) {
+      double mu_abs = fabs(mu);
+      if (mu_abs > 0 && max_b > 0 && (mu_abs >= TR_BND || max_b >= TR_BND)) {
          in_float = 0;
       }
+      else {
+         max_a += mu_abs*max_b;
+         if (max_a >= TR_BND)
+            in_float = 0;
+      }
    }
 
    if (in_float) {
@@ -545,7 +562,7 @@ void GivensComputeGS(double **B1, double **mu, double **aux, long k, long n,
    if (k > n) p[k] = 0;
 
    for (i = 1; i <= k; i++)
-      if (!IsFinite(&p[i])) Error("G_LLL_FP...numbers too big");
+      CheckFinite(&p[i]);
 }
 
 static double red_fudge = 0;
@@ -790,7 +807,7 @@ long ll_G_LLL_FP(mat_ZZ& B, mat_ZZ* U, double delta, long deep,
    
                conv(MU, mu1);
 
-               RowTransform(B(k), B(j), MU, mu1, B1[k], B1[j], in_vec,
+               RowTransform(B(k), B(j), MU, B1[k], B1[j], in_vec,
                             max_b[k], max_b[j], in_float);
                if (U) RowTransform((*U)(k), (*U)(j), MU);
             }
@@ -923,15 +940,12 @@ long G_LLL_FP(mat_ZZ& B, mat_ZZ* U, double delta, long deep,
    }
 
    for (i = 1; i <=m; i++)
-      for (j = 1; j <= n; j++) 
+      for (j = 1; j <= n; j++) {
          conv(B1[i][j], B(i, j));
+         CheckFinite(&B1[i][j]);
+      }
 
          
-   for (i = 1; i <= m; i++) 
-      for (j = 1; j <= n; j++)
-         if (!IsFinite(&B1[i][j]))
-             Error("G_LLL_FP: numbers too big...use G_LLL_XD");
-
    GivensCache_FP cache(m, n);
 
    new_m = ll_G_LLL_FP(B, U, delta, deep, check, B1, mu, aux, m, 1, quit, cache);
@@ -1080,8 +1094,9 @@ void ComputeG_BKZThresh(double *c, long beta)
 }
 
 static 
-void G_BKZStatus(double tt, double enum_time, long NumIterations, 
-               long NumTrivial, long NumNonTrivial, long NumNoOps, long m, 
+void G_BKZStatus(double tt, double enum_time, unsigned long NumIterations, 
+               unsigned long NumTrivial, unsigned long NumNonTrivial, 
+               unsigned long NumNoOps, long m, 
                const mat_ZZ& B)
 {
    cerr << "---- G_BKZ_FP status ----\n";
@@ -1250,8 +1265,7 @@ long G_BKZ_FP(mat_ZZ& BB, mat_ZZ* UU, double delta,
    for (i = 1; i <=m; i++)
       for (j = 1; j <= n; j++) {
          conv(B1[i][j], B(i, j));
-         if (!IsFinite(&B1[i][j])) 
-            Error("G_BKZ_FP: numbers too big...use G_BKZ_XD");
+         CheckFinite(&B1[i][j]);
       }
 
          
@@ -1262,10 +1276,10 @@ long G_BKZ_FP(mat_ZZ& BB, mat_ZZ* UU, double delta,
    double tt;
 
    double enum_time = 0;
-   long NumIterations = 0;
-   long NumTrivial = 0;
-   long NumNonTrivial = 0;
-   long NumNoOps = 0;
+   unsigned long NumIterations = 0;
+   unsigned long NumTrivial = 0;
+   unsigned long NumNonTrivial = 0;
+   unsigned long NumNoOps = 0;
 
    long verb = verbose;
 
@@ -1319,7 +1333,7 @@ long G_BKZ_FP(mat_ZZ& BB, mat_ZZ* UU, double delta,
 
          for (i = jj; i <= kk; i++) {
             c[i] = mu[i][i]*mu[i][i];
-            if (!IsFinite(&c[i])) Error("numbers too big...use G_BKZ_XD");
+            CheckFinite(&c[i]);
          }
 
          if (prune > 0)
@@ -1475,8 +1489,10 @@ long G_BKZ_FP(mat_ZZ& BB, mat_ZZ* UU, double delta,
                   tp = B1[i-1]; B1[i-1] = B1[i]; B1[i] = tp;
                }
       
-               for (i = 1; i <= n; i++)
+               for (i = 1; i <= n; i++) {
                   conv(B1[jj][i], B(jj, i));
+                  CheckFinite(&B1[jj][i]);
+               }
 
                if (IsZero(B(jj))) Error("G_BKZ_FP: internal error");
       
