@@ -55,6 +55,9 @@ void InnerProduct(zz_p& x, const vec_zz_p& a, const vec_zz_p& b)
 void InnerProduct(zz_p& x, const vec_zz_p& a, const vec_zz_p& b,
                   long offset)
 {
+   if (offset < 0) Error("InnerProduct: negative offset");
+   if (NTL_OVERFLOW(offset, 1, 0)) Error("InnerProduct: offset too big");
+
    long n = min(a.length(), b.length()+offset);
    long i;
    zz_p accum, t;
@@ -236,7 +239,7 @@ zz_p operator*(const vec_zz_p& a, const vec_zz_p& b)
 void VectorCopy(vec_zz_p& x, const vec_zz_p& a, long n)
 {
    if (n < 0) Error("VectorCopy: negative length");
-   if (n >= (1L << (NTL_BITS_PER_LONG-4))) Error("overflow in VectorCopy");
+   if (NTL_OVERFLOW(n, 1, 0)) Error("overflow in VectorCopy");
 
    long m = min(n, a.length());
 
